% johan.m: Performs Johansen test for cointegration. Produces
%            likelihood ratio statistic for testing the null of
%            no cointegrating vector against the alternative of
%            one cointegrating vector.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load data

  load def.dat;
  p = def;

  load u.dat;

  T = length(u);

% adjust sample size if necessary

%  p = p(41:T);

%  u = u(41:T);

%  T = length(u);

% compute first difference

  dp = p(2:T) - p(1:T-1);

  du = u(2:T) - u(1:T-1);

% estimate first set of auxiliary regressions

  yt = dp(2:T-1);

  xt = [ dp(1:T-2) du(1:T-2) ];

  pi11 = inv(xt'*xt)*xt'*yt;

  ut1 = yt - xt*pi11;

  yt = du(2:T-1);

  xt = [ dp(1:T-2) du(1:T-2) ];

  pi12 = inv(xt'*xt)*xt'*yt;

  ut2 = yt - xt*pi12;

  pi1 = [ pi11' ; pi12' ];

  ut = [ ut1 ut2 ];

% estimate second set of auxiliary regressions

  yt = p(2:T-1);

  xt = [ dp(1:T-2) du(1:T-2) ];

  thet11 = inv(xt'*xt)*xt'*yt;

  vt1 = yt - xt*thet11;

  yt = u(2:T-1);

  xt = [ dp(1:T-2) du(1:T-2) ];

  thet12 = inv(xt'*xt)*xt'*yt;

  vt2 = yt - xt*thet12;

  thet1 = [ thet11' ; thet12' ];

  vt = [ vt1 vt2 ];

% calculate covariance matrices

  siguu = (1/(T-2))*ut'*ut;

  siguv = (1/(T-2))*ut'*vt;

  sigvu = siguv';

  sigvv = (1/(T-2))*vt'*vt;

% find eigenvalues and eigenvectors

  bigsig = inv(sigvv)*sigvu*inv(siguu)*siguv;

  [amat,lammat] = eig(bigsig);

  [lams,lamord] = sort(diag(lammat));

  amat = amat(:,lamord);

  amat(:,1) = amat(:,1)/(sqrt(amat(:,1)'*sigvv*amat(:,1)));

  amat(:,2) = amat(:,2)/(sqrt(amat(:,2)'*sigvv*amat(:,2)));

% calculate likelihood ratio statistic

  lrstat = -(T-2)*log(1-lams(2));

% calculate parameter estimates

  avec = amat(:,2);

  xio = siguv*avec*avec';

  xi1 = pi1 - xio*thet1;

  et = ut - vt*xio';

  omeg = (1/(T-2))*et'*et;

  