% philper.m: Performs Phillips-Perron unit root test.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load data

  load u.dat;
  yt = u;

%  load cpi.dat;
%  yt = cpi;

%  load core.dat;
%  yt = core;

%  load def.dat;
%  yt = def;

  T = length(yt) - 1;

% adjust sample size if necessary

% yt = yt(41:T+1);

% T = length(yt) - 1;

% set up matrices for regression

  xt = [ ones(T,1) yt(1:T) ];

  yt = yt(2:T+1);

% perform regression and compute preliminary statistics

  rhovec = inv(xt'*xt)*xt'*yt;

  ut = yt - xt*rhovec;

  s2 = ut'*ut/(T-2);

  s = sqrt(s2);

  sigrvec = s2*inv(xt'*xt);

  rho = rhovec(2);

  sigr = sqrt(sigrvec(2,2));

  tstat = (rho-1)/sigr;

% compute Newey-West variance

  gamo = (1/T)*ut'*ut;

  gamvec = zeros(50,1);

  for j = 1:51

    gamu = ut(j+1:T);

    gamuj = ut(1:T-j);

    gamvec(j) = (1/T)*gamu'*gamuj;

  end

% compute Phillips-Perron statistic for various values of q

  ppstat = zeros(51,1);

  ppstat(1) = tstat;

  for q = 1:50

    lam2 = gamo;

    for j = 1:q

      lam2 = lam2 + 2*(1-j/(q+1))*gamvec(j);

    end

    lam = sqrt(lam2);

    ppstat(q+1) = (sqrt(gamo/lam2))*tstat ...
                    - (1/2)*((lam2-gamo)/lam)*(T*sigr/s);

  end

% compute Andrews' optimal q

  pihat = inv(ut(1:T-1)'*ut(1:T-1))*ut(1:T-1)'*ut(2:T);

  qstar = 4*pihat^2/((1-pihat)^2*(1+pihat)^2);
