function llfn = unllfnse(bigthet)
% Uses the Kalman filter to evaluate the negative log likelihood
%   function for the unconstrained version of the Barro-Gordon
%   model when the natural rate of unemployment follows an AR(1)
%   in first differences. Does not use transformed parameters and
%   can therefore be used to compute standard errors.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% define variables

  global ut pit

  T = length(ut) - 1;

  gam = bigthet(1);
  ppp = bigthet(2);
  ppu = bigthet(3);
  pup = bigthet(4);
  puu = bigthet(5);
  tpp1 = bigthet(6);
  tpu1 = bigthet(7);
  tup1 = bigthet(8);
  tuu1 = bigthet(9);
  tpp2 = bigthet(10);
  tpu2 = bigthet(11);
  tup2 = bigthet(12);
  tuu2 = bigthet(13);
  sigp = bigthet(14);
  sigu = bigthet(15);
  sigpu = bigthet(16);  

% construct state-space model

  yt = [ pit(2:T+1)-gam*ut(2:T+1) ut(2:T+1)-ut(1:T) ];

  SIG = [ sigp^2 sigpu ; sigpu sigu^2 ];

  F = [ ppp ppu tpp1 tpu1 tpp2 tpu2 ; ...
        pup puu tup1 tuu1 tup2 tuu2 ; ...
        0 0 0 0 0 0 ; ...
        0 0 0 0 0 0 ; ...
        0 0 1 0 0 0 ; ...
        0 0 0 1 0 0 ];

  Q = [ 1 0 ; 0 1 ; 1 0 ; 0 1 ; 0 0 ; 0 0 ];

  H = [ eye(2) zeros(2,4) ];

  QSQ = Q*SIG*Q';

% initialize Kalman filter

  xit = zeros(6,1);

  Pt = inv(eye(36)-kron(F,F))*QSQ(:);

  Pt = reshape(Pt,6,6);

% evaluate negative log likelihood function

  llfn =  T*log(2*pi);

  for t = 1:T

    et = yt(t,:)' - H*xit;

    llfn = llfn + (1/2)*log(det(H*Pt*H')) + (1/2)*et'*inv(H*Pt*H')*et;

    Kt = F*Pt*H'*inv(H*Pt*H');

    xit = F*xit + Kt*et;

    Pt = (F-Kt*H)*Pt*(F'-H'*Kt') + QSQ;

  end
